﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Navigation;

namespace HamburgerMenuNavigationUWP
{
  /// <summary>
  /// Provides application-specific behavior to supplement the default Application class.
  /// </summary>
  sealed partial class App : Application
  {
    bool m_FirstNavigation = true;

    // The current page entry
    PageStackEntry m_CurrentPage = null;
    // The backstack
    List<PageStackEntry> m_BackStack = new List<PageStackEntry>();
    // Pages with NavigationCacheMode == NavigationCacheMode.Required
    Dictionary<string, Pages.BasePage> m_PagesCache = new Dictionary<string, Pages.BasePage>();
    bool m_GoingBack = false;

    /// <summary>
    /// Initializes the singleton application object.  This is the first line of authored code
    /// executed, and as such is the logical equivalent of main() or WinMain().
    /// </summary>
    public App()
    {
      this.InitializeComponent();
      this.Suspending += OnSuspending;
    }

    /// <summary>
    /// Invoked when the application is launched normally by the end user.  Other entry points
    /// will be used such as when the application is launched to open a specific file.
    /// </summary>
    /// <param name="e">Details about the launch request and process.</param>
    protected override void OnLaunched(LaunchActivatedEventArgs e)
    {
#if DEBUG
      if (System.Diagnostics.Debugger.IsAttached) {
        this.DebugSettings.EnableFrameRateCounter = true;
      }
#endif
      Frame rootFrame = Window.Current.Content as Frame;

      // Do not repeat app initialization when the Window already has content,
      // just ensure that the window is active
      if (rootFrame == null) {
        // Create a Frame to act as the navigation context and navigate to the first page
        rootFrame = new Frame();

        rootFrame.Navigated += RootFrame_Navigated;
        rootFrame.Navigating += RootFrame_Navigating;
        rootFrame.NavigationFailed += OnNavigationFailed;

        if (e.PreviousExecutionState == ApplicationExecutionState.Terminated) {
          //TODO: Load state from previously suspended application
        }

        // Place the frame in the current Window
        Window.Current.Content = rootFrame;

        // Register a handler for BackRequested events and set the
        // visibility of the Back button
        SystemNavigationManager.GetForCurrentView().BackRequested += OnBackRequested;

        SystemNavigationManager.GetForCurrentView().AppViewBackButtonVisibility =
            rootFrame.CanGoBack ?
            AppViewBackButtonVisibility.Visible :
            AppViewBackButtonVisibility.Collapsed;
      }

      if (e.PrelaunchActivated == false) {
        if (rootFrame.Content == null) {
          // When the navigation stack isn't restored navigate to the first page,
          // configuring the new page by passing required information as a navigation
          // parameter
          rootFrame.Navigate(typeof(MainPage), e.Arguments);
        }
        // Ensure the current window is active
        Window.Current.Activate();
      }
    }

    private async void RootFrame_Navigated(object sender, NavigationEventArgs e)
    {
      // Navigate to the start page:
      MainPage mp = (Window.Current.Content as Frame).Content as MainPage;
      if (mp != null && m_FirstNavigation) {
        m_FirstNavigation = false;
        await mp.Frame.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async () =>
        {
          await System.Threading.Tasks.Task.Delay(10);
          Type sp = Type.GetType(MainPage.StartPage);
          mp.Frame.Navigate(sp);
        });
      }
    }

    private void OnBackRequested(object sender, BackRequestedEventArgs e)
    {
      // Check if the current page wants to manage the back button:
      MainPage mp = (Window.Current.Content as Frame).Content as MainPage;
      if (mp != null) {
        if (mp.MainGrid.Children.Count > 0) {
          Pages.BasePage curPage = mp.MainGrid.Children[0] as Pages.BasePage;
          if (curPage != null) {
            curPage.OnBackRequested(sender, e);
            if (e.Handled)
              return;
          }
        }
      }

      // Navigate to the previous page if possible:
      Frame rootFrame = Window.Current.Content as Frame;

      if (m_BackStack.Count > 0) {
        e.Handled = true;

        PageStackEntry pse = m_BackStack.Last();
        m_BackStack.Remove(pse);
        m_GoingBack = true;
        rootFrame.Navigate(pse.SourcePageType, pse.Parameter, pse.NavigationTransitionInfo);
      }
    } // OnBackRequested

    private void RootFrame_Navigating(object sender, NavigatingCancelEventArgs e)
    {
      if (e.SourcePageType == typeof(MainPage))
        return;

      MainPage mp = (Window.Current.Content as Frame).Content as MainPage;

      // Do not navigate to the same page:
      if (m_CurrentPage != null && e.SourcePageType == m_CurrentPage.SourcePageType){
        if (mp != null)
          mp.IsPaneOpen = false;
        e.Cancel = true;
        return;
      }

      Pages.BasePage oldPage = null;
      Pages.BasePage newPage = null;

      //Check the cache:
      if (!m_PagesCache.TryGetValue(e.SourcePageType.FullName, out newPage))
        newPage = (Pages.BasePage)Activator.CreateInstance(e.SourcePageType, new object[] { });

      if (newPage != null) {
        newPage.Transitions = new TransitionCollection();   
        if (mp != null) {
          if (mp.MainGrid.Children.Count > 0)
            oldPage = mp.MainGrid.Children[0] as Pages.BasePage;

          // Call page events:
          if (oldPage != null) {
            oldPage.OnNavigatingFrom(e);
            if (e.Cancel)
              return;
          }

          // Animations:
          if (m_GoingBack) {
            if (oldPage != null) { 
              oldPage.Transitions = new TransitionCollection();
              if (oldPage.BackExitTransition != null)
                oldPage.Transitions = new TransitionCollection() { oldPage.BackExitTransition };              
            }

            if (newPage.BackEnterTransition != null)
              newPage.Transitions = new TransitionCollection() { newPage.BackEnterTransition };
          } else {
            if (oldPage != null) {
              oldPage.Transitions = new TransitionCollection();
              if (oldPage.ForwardExitTransition != null)
                oldPage.Transitions = new TransitionCollection() { oldPage.ForwardExitTransition };
            }

            if (newPage.ForwardEnterTransition != null)
              newPage.Transitions = new TransitionCollection() { newPage.ForwardEnterTransition };
          }
          
          if (oldPage != null) {
            // Remove oldPage from the MainPage:
            mp.MainGrid.Children.Remove(oldPage);

            // Call oldPage events:
            oldPage.OnNavigatedFrom(new Pages.NavigationArgs() { NavigationMode = m_GoingBack ? NavigationMode.Back : NavigationMode.Forward });
          }
            
          // Add the new page to the MainPage:
          mp.MainGrid.Children.Add(newPage);

          // Call newPage events:
          newPage.OnNavigatedTo(new Pages.NavigationArgs() { NavigationMode = m_GoingBack ? NavigationMode.Back : NavigationMode.New, Parameter = e.Parameter, SourcePageType = e.SourcePageType });

          // Select the menu item
          if (!mp.SelectMenu(e.SourcePageType.FullName) && !string.IsNullOrEmpty(newPage.SelectMenuAction))
            mp.SelectMenu(newPage.SelectMenuAction);

          // Close the menu:
          mp.IsPaneOpen = false;

          // Add the new page to the back stack
          if (!m_GoingBack) {
            if (m_CurrentPage != null)
              m_BackStack.Add(m_CurrentPage);            
          }
          m_CurrentPage = new PageStackEntry(e.SourcePageType, e.Parameter, e.NavigationTransitionInfo);

          // Save/remove pages with NavigationCacheMode.Required
          if (oldPage != null) {
            if (!m_GoingBack && oldPage.NavigationCacheMode == NavigationCacheMode.Required) {
              m_PagesCache[oldPage.GetType().FullName] = oldPage;
            } else if (m_GoingBack && m_PagesCache.ContainsKey(oldPage.GetType().FullName)) {
              m_PagesCache.Remove(oldPage.GetType().FullName);
            }
          }

          // Show/hide the back button
          SystemNavigationManager.GetForCurrentView().AppViewBackButtonVisibility =
            m_BackStack.Count > 0 ? AppViewBackButtonVisibility.Visible : AppViewBackButtonVisibility.Collapsed;

          m_GoingBack = false;
        }
      }
      e.Cancel = true;
    } // RootFrame_Navigating

    /// <summary>
    /// Invoked when Navigation to a certain page fails
    /// </summary>
    /// <param name="sender">The Frame which failed navigation</param>
    /// <param name="e">Details about the navigation failure</param>
    void OnNavigationFailed(object sender, NavigationFailedEventArgs e)
    {
      throw new Exception("Failed to load Page " + e.SourcePageType.FullName);
    }

    /// <summary>
    /// Invoked when application execution is being suspended.  Application state is saved
    /// without knowing whether the application will be terminated or resumed with the contents
    /// of memory still intact.
    /// </summary>
    /// <param name="sender">The source of the suspend request.</param>
    /// <param name="e">Details about the suspend request.</param>
    private void OnSuspending(object sender, SuspendingEventArgs e)
    {
      var deferral = e.SuspendingOperation.GetDeferral();
      //TODO: Save application state and stop any background activity
      deferral.Complete();
    }
  }
}
