﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;

namespace Controls
{
  public sealed class MenuButton : Control
  {
    const int SlowAnimation = 150;
    const int FastAnimation = 50;

    TextBlock m_Label = null;
    TextBlock m_Icon = null;
    Grid m_Container = null;
    
    public MenuButton()
    {
      this.DefaultStyleKey = typeof(MenuButton);
    }

    #region Dependency Properties
    public string Label
    {
      get { return (string)this.GetValue(LabelProperty); }
      set { this.SetValue(LabelProperty, value); }
    }
    public static readonly DependencyProperty LabelProperty = DependencyProperty.Register(
      "Label", typeof(string), typeof(MenuButton), new PropertyMetadata(string.Empty, LabelChanged));

    private static void LabelChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      MenuButton This = d as MenuButton;
      if (This != null)
        This.Set();
    }

    public string Icon
    {
      get { return (string)this.GetValue(IconProperty); }
      set { this.SetValue(IconProperty, value); }
    }
    public static readonly DependencyProperty IconProperty = DependencyProperty.Register(
      "Icon", typeof(string), typeof(MenuButton), new PropertyMetadata(string.Empty, IconChanged));

    private static void IconChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      MenuButton This = d as MenuButton;
      if (This != null)
        This.Set();
    }

    public bool Selected
    {
      get { return (bool)this.GetValue(SelectedProperty); }
      set { this.SetValue(SelectedProperty, value); }
    }
    public static readonly DependencyProperty SelectedProperty = DependencyProperty.Register(
      "Selected", typeof(bool), typeof(MenuButton), new PropertyMetadata(false, SelectedChanged));

    private static void SelectedChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      MenuButton This = d as MenuButton;
      bool value = (bool)e.NewValue;
      if (This != null) {
        if (value) {
          This.Foreground = This.SelectedForegroundBrush;
          This.AnimateBackgroundColor(This.SelectedColor, FastAnimation);
        } else {
          This.Foreground = This.ForegroundBrush;
          This.AnimateBackgroundColor(This.BackgroundColor, FastAnimation);
        }
      }        
    }

    public Windows.UI.Color BackgroundColor
    {
      get { return (Windows.UI.Color)this.GetValue(BackgroundColorProperty); }
      set { this.SetValue(BackgroundColorProperty, value); }
    }
    public static readonly DependencyProperty BackgroundColorProperty = DependencyProperty.Register(
      "BackgroundColor", typeof(Windows.UI.Color), typeof(MenuButton),
      new PropertyMetadata(Windows.UI.Colors.Transparent));

    public Windows.UI.Color HoverColor
    {
      get { return (Windows.UI.Color)this.GetValue(HoverColorProperty); }
      set { this.SetValue(HoverColorProperty, value); }
    }
    public static readonly DependencyProperty HoverColorProperty = DependencyProperty.Register(
      "HoverColor", typeof(Windows.UI.Color), typeof(MenuButton), 
      new PropertyMetadata((Windows.UI.Color)Application.Current.Resources["SystemListLowColor"]));

    public Windows.UI.Color PressedColor
    {
      get { return (Windows.UI.Color)this.GetValue(PressedColorProperty); }
      set { this.SetValue(PressedColorProperty, value); }
    }
    public static readonly DependencyProperty PressedColorProperty = DependencyProperty.Register(
      "PressedColor", typeof(Windows.UI.Color), typeof(MenuButton),
      new PropertyMetadata((Windows.UI.Color)Application.Current.Resources["SystemListMediumColor"]));

    public Windows.UI.Color SelectedColor
    {
      get { return (Windows.UI.Color)this.GetValue(SelectedColorProperty); }
      set { this.SetValue(SelectedColorProperty, value); }
    }
    public static readonly DependencyProperty SelectedColorProperty = DependencyProperty.Register(
      "SelectedColor", typeof(Windows.UI.Color), typeof(MenuButton),
      new PropertyMetadata((Windows.UI.Color)Application.Current.Resources["SystemAccentColor"]));

    public SolidColorBrush ForegroundBrush
    {
      get { return (SolidColorBrush)this.GetValue(ForegroundBrushProperty); }
      set { this.SetValue(ForegroundBrushProperty, value); }
    }
    public static readonly DependencyProperty ForegroundBrushProperty = DependencyProperty.Register(
      "ForegroundBrush", typeof(SolidColorBrush), typeof(MenuButton),
      new PropertyMetadata((SolidColorBrush)Application.Current.Resources["SystemControlForegroundBaseHighBrush"]));

    public SolidColorBrush SelectedForegroundBrush
    {
      get { return (SolidColorBrush)this.GetValue(SelectedForegroundBrushProperty); }
      set { this.SetValue(SelectedForegroundBrushProperty, value); }
    }
    public static readonly DependencyProperty SelectedForegroundBrushProperty = DependencyProperty.Register(
      "SelectedForegroundBrush", typeof(SolidColorBrush), typeof(MenuButton),
      new PropertyMetadata((SolidColorBrush)Application.Current.Resources["SystemControlForegroundChromeWhiteBrush"]));
    #endregion

    public string ActionName { get; set; }

    #region Pointer events
    private void OnPointerEntered(System.Object sender, PointerRoutedEventArgs e)
    {
      if (!Selected)
        AnimateBackgroundColor(HoverColor, SlowAnimation);
   }

    private void OnPointerExited(System.Object sender, PointerRoutedEventArgs e)
    {
      AnimateBackgroundColor(GetBackground(), SlowAnimation);
    }

    private void OnPointerPressed(System.Object sender, PointerRoutedEventArgs e)
    {
      AnimateBackgroundColor(PressedColor, FastAnimation);
      Foreground = ForegroundBrush;
    }

    private void OnPointerReleased(System.Object sender, PointerRoutedEventArgs e)
    {
      if (Selected) {
        Foreground = SelectedForegroundBrush;
        AnimateBackgroundColor(SelectedColor, FastAnimation);
      } else
        AnimateBackgroundColor(HoverColor, FastAnimation);
    }
    #endregion

    private Windows.UI.Color GetBackground()
    {
      if (Selected)
        return SelectedColor;
      return BackgroundColor;
    }

    private void SetColors()
    {
      Foreground = ForegroundBrush;
      if (Selected)
        Background = new SolidColorBrush(SelectedColor);
      else
        Background = new SolidColorBrush(BackgroundColor);
    } // SetColors

    private void Set()
    {
      if (m_Icon != null)
        m_Icon.Text = Icon;

      if (m_Label != null)
        m_Label.Text = Label;
    } // Set

    private void AnimateBackgroundColor(Windows.UI.Color to, int milliseconds)
    {
      if (m_Container == null)
        return;

      Windows.UI.Color from = m_Container.Background != null ? (m_Container.Background as SolidColorBrush).Color : Windows.UI.Colors.Transparent;
      Storyboard sb = new Storyboard();
      ColorAnimation anim = new ColorAnimation()
      {
        From = from,
        To = to,
        Duration = new Duration(TimeSpan.FromMilliseconds(milliseconds))
      };

      Storyboard.SetTarget(anim, m_Container.Background);
      Storyboard.SetTargetProperty(sb, "Color");
      sb.Children.Add(anim);
      sb.Begin();
    } // AnimateBackgroundColor

    public void Select()
    {
      SplitView view = Utility.GetParent<SplitView>(this);
      if (view != null) {
        foreach (MenuButton btn in Utility.GetChildren<MenuButton>(view)) {
          if (btn != this)
            btn.Selected = false;
        }
      }
      Selected = true;
    } // Select

    protected override void OnApplyTemplate()
    {
      base.OnApplyTemplate();

      m_Container = GetTemplateChild("m_Container") as Grid;
      if (m_Container != null) {
        if (Selected)
          m_Container.Background = new SolidColorBrush(SelectedColor);
        else
          m_Container.Background = new SolidColorBrush(BackgroundColor);
        m_Container.PointerPressed += OnPointerPressed;
        m_Container.PointerReleased += OnPointerReleased;
        m_Container.PointerEntered += OnPointerEntered;
        m_Container.PointerExited += OnPointerExited;
      }

      m_Label = GetTemplateChild("m_Label") as TextBlock;
      m_Icon = GetTemplateChild("m_Icon") as TextBlock;

      SetColors();
      Set();
    } // OnApplyTemplate

  }
}
